#!/usr/bin/env python

"""Lumberjack v1.0 -- a simple logging facility for Python programs

This class is generally meant to be subclassed for specific
applications, but it can be used directly.

To add a log entry, add() one of the LogObject classes
to the Lumberjack class like so:

my_lumberjack.add(Log('some text'))

Any hook function *must* take 1 string argument!  This will be the
LogObject.item attribute, which contains the full text of the 'thing'
to be logged.  If a hook returns a value (not None) then the
LogObject.item will be replaced with that value.

This allows you to use external hook functions to modify logging items
on the fly.

Be sure to have the hook function return None if you don't want to
change the item.

This module is safe to use with 'from lumberjack import *'

Copyleft (C) 1999 Preston Landers <preston@askpreston.com>
"""

###
### TODO:
###  -- kernel syslog support??
###

import time
from types import *

# event logging facility
class Lumberjack:
    def __init__(self, LogFile, Standard_Output = None, Keep_Internal_Log = None, Hook_Func = None, Debugging = None):
        """__init__ parameters for Lumberjack:
        
LogFile = string containing valid filename for log to be opened for append
Standard_Output = if not None, also print log to stdout
Keep_Internal_Log = if not None, also keep an internal list (Lumberjack.log_list) of all log entries.
  Warning!  This parameter should be used with great care!  There are (currently) no safety valves on
  log size.  If a program is run in daemon mode, this buffer will continue to expand forever.  A busy
  system could quickly run out of memory.  Only use this if you know what you're doing.
Hook_Func = a function or a list of functions to be called for each add() operation
Debugging = if not None, debugging/warning statements are printed. defaults to None"""
        if Debugging:
            self.debug = "on"
        else:
            # self.debug = None
            self.debug = "on"
        
        if Standard_Output:
            self.stdoutput = "on"
        else:
            self.stdoutput = None

        if Keep_Internal_Log:
            self.log_list = []  # empty list of lines (log stored in memory)
        else:
            self.log_list = None  # we don't want the log stored in memory

        self.LogFile = LogFile
        if LogFile:
            try:
                self.log = open(self.LogFile, 'a')  # open for append
            except:
                print "There was a problem opening the log file for append; exiting."
                raise SystemExit(1)

        # install hooks here
        self.hooks = None
        if Hook_Func:
            self.add_hook(Hook_Func)
        
    def __del__(self):
        if self.LogFile:
            self.log.close()

    def debug(self, Debug = None):
        """debug(status) -- toggles debugging.  If status == None, debugging is off. Otherwise, it is on.
        
In general, having debugging on means some extra warning messages will be printed to the console."""
        if Debug:
            self.debug = "on"
        else:
            self.debug = None

    def add_hook(self, Hook_Func):
        """add_hook(hook_function) -- installs a 'hook' function for this Lumberjack.
        
Can be a single hook function or a list of hooks.  Each hook will be called once for each
log entry with the LogObject.item as its only parameter."""
        if not self.hooks:
            self.hooks = []
            
        if type(Hook_Func) is ListType:
            # TODO: check items in list for FunctionType
            for Hook in Hook_Func:
                if type(Hook) is not FunctionType and type(Hook) is not MethodType:
                    if self.debug:
                        print "Invalid Hook type provided in Lumberjack.add_hook()."
                else:
                    self.hooks.append(Hook)
        elif type(Hook_Func) is FunctionType or type(Hook_Func) is MethodType:
            # print "Added a hook."
            self.hooks.append(Hook_Func)
        else:
            if self.debug:
                print "Invalid Hook type provided in Lumberjack.add_hook()."
            # self.hooks = None

    def remove_hook(self, Obsolete_Hook = None):        
        """remove_hook(the_hook) -- removes an individual hook function from the Lumberjack's
list of hooks.

Must be called individually for each hook that should be removed.  If a hook
is requested to be removed but is not present, the error will pass silently unless debugging
is enabled (in which case a warning will be printed to stdout.)

If called with None, all hooks will be deleted silently!  A warning will be printed if
debugging is enabled.
"""
        if not Obsolete_Hook:
            self.hooks = None
            if self.debug:
                print "Removed all hooks in Lumberjack; continuing."
        elif self.hooks:
            try:
                self.hooks.remove(Obsolete_Hook)
            except:
                if self.debug:
                    print "Tried to remove a hook in Lumberjack that wasn't present; continuing."
            
    # use the add method to add a log entry
    def add(self, object):
        """add(Log_Object) -- actually adds an entry to the log.
Log_Object must be an object subclassed from Lumberjack.LogObject.
Hooks will be applied in the order they were installed.  Hooks *may* modify the object
to be added."""
        # apply hooks before anything else
        if self.hooks:
            # print "Trying to access hook"
            for Hook in self.hooks:
                result = Hook(object.item)
                # result = apply(Hook, (object.item))
                if result:
                    object.item = result
                    if self.debug:
                        print "A hook modified an item in Lumberjack."
                    
        # it can print to the log file
        if self.LogFile:
            self.log.write(str(object.item) + '\n')
        # and/or standard output
        if self.stdoutput:
            print str(object.item)
        # and/or to an internal representation of the log
        if self.log_list != None:
            ## don't think we want a newline in the internal representation
            # self.log_list.append(str(object.item) + '\n')
            self.log_list.append(str(object.item))

    class LogObject:
        """LogObject is the class you will want to extend from to add logging event handlers
for different types of items."""
        def __init__(self):
            self.item = ""

    class Log(LogObject):
        """Log() -- A simple, generic log object.
use like this:
my_log.add(Log('hello'))"""
        def __init__(self, a_string):
            Lumberjack.LogObject.__init__(self)            
            self.item = self.item + a_string


if __name__ == "__main__":
    print "This is the Lumberjack test suite."

    logfile = "./test.log"
    
    print "Creating a log object that will:"
    # print " - append to %s" % logfile
    print " - print to stdout"
    print " - keep a log in memory"
    # print " - print debugging statements"
    print " - have no (initial) hooks"
    
    log = Lumberjack(None, "stdout", "internal_log", None, None)

    print "Now logging a few simple items:"

    log.add(log.Log("Item 1: apathy"))
    log.add(log.Log("Item 2: boisterousness"))
    log.add(log.Log("Item 3: carelessness"))

    print "Now installing hook functions."

    import string
    
    def Hook_One(item):
        """convert to uppercase"""
        return string.upper(item)

    def Hook_Two(item):
        print "Inside a hook; nothing done on %s" % item
        return

    def Hook_Three(item):
        """append something to each item"""
        return item + " -- Timber Ho!"

    log.add_hook([Hook_One, Hook_Two, Hook_Three])

    print "Now testing hooks:"
    
    log.add(log.Log("Item 4: dispair"))
    log.add(log.Log("Item 5: ennui"))
    log.add(log.Log("Item 6: fright"))    

    print "Now removing first hook."

    log.remove_hook(Hook_One)

    log.add(log.Log("Item 7: generosity"))

    print "Now removing all hooks."

    log.remove_hook()

    log.add(log.Log("Item 8: helplessness"))
    
    print "Printing internal representation:"

    if log.log_list:
        for item in log.log_list:
            print item
    else:
        print "No internal log kept."

    print "Finished."
